/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.3
import TableModel 0.1

//Table view showing the printer table
TableView {
    id: availablePrintersView
    width: 110
    height: 160
    anchors.fill: parent
    anchors.rightMargin: 25
    columnSpacing: 0
    rowSpacing: 0
    clip: true
    model: available_printers_model
    //Translated header fields
    property var headerFields: {
        "name": qsTr("name"),
        "server": qsTr("server"),
        "location": qsTr("location"),
        "driver": qsTr("driver"),
        "format": qsTr("format"),
        "color": qsTr("color"),
        "duplex": qsTr("duplex"),
        "staple": qsTr("staple"),
        "punch": qsTr("punch"),
        "booklet": qsTr("booklet"),
        "scanner": qsTr("scanner"),
        "opts": qsTr("opts"),
        "comment": qsTr("comment")
    }
    //Object storing icon names
    property var iconColumns: {
        "color": {
            "color": {
                "icon": "color.svg",
                "tooltip": qsTr("Color")
            },
            "grayscale": {
                "icon": "grayscale.svg",
                "tooltip": qsTr("Grayscale")
            }
        }
    }

    topMargin: columnsHeader.implicitHeight
    property var columnwidths: available_printers_model.getColumnWidths();
    columnWidthProvider: function (column) {
        return columnwidths[column];
    }
    rowHeightProvider: function(row) {return 35;}
    reuseItems: false


    //On load: Load printer list from smb
    Component.onCompleted: {
        //Perform self check
        try {
            controller.selfCheck();
        } catch (e) {
            var messages = e.message.split(';');
            var translatedErrors = '';
            for (var i = 0; i < messages.length; i++) {
                if (translatedErrors.length > 0) {
                    translatedErrors += '\n';
                }
                translatedErrors += qsTr(messages[i]);
            }

            mainErrorDialog.setContent(translatedErrors);
            mainErrorDialog.open();
            return;
        }
        //Check pritner list and driver
        var list = available_printers_model.loadLocalPrinterDefinitions();
        var driver = controller.isDriverLoaded();
        //Try to get a local printer definition list
        if (!(list && driver)) {
            //If not available, load from remote
            if (available_printers_model.needCredentialsForPrinterList()) {
                var dialogMode;
                if (list) {
                    if(!driver) {
                        dialogMode = SMBDialog.Mode.DRIVER;
                    }
                } else {
                    if (driver) {
                        dialogMode = SMBDialog.Mode.PRINTER_LIST;
                    } else {
                        dialogMode = SMBDialog.Mode.ALL;
                    }
                }

                smbDialog.mode = dialogMode;
                smbDialog.open();
            } else {
                available_printers_model.readRemotePrinterListAnonymous();
            }
        }
    }

    //Header row
    Row {
        id: columnsHeader
        y: availablePrintersView.contentY
        z: 2
        Repeater {
            id: headerRepeater
            model: availablePrintersView.columns > 0 ? availablePrintersView.columns : 1
            Label {
                width: availablePrintersView.columnWidthProvider(modelData) + 1
                height: availablePrintersView.rowHeightProvider(0);
                text: headerFields[available_printers_model.headerData(modelData, Qt.Horizontal)]
                font.pixelSize: 15
                color: styleUtil.getFontColor()
                padding: 10
                elide: Text.ElideRight
                verticalAlignment: Text.AlignVCenter
                background: Rectangle { color: styleUtil.getTableHeaderColor() }

                property alias sortButton: sortButton

                SortButton {
                    id: sortButton
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.topMargin: -5
                    anchors.rightMargin: 10
                    column: modelData
                    onClicked: {
                        for (var i = 0; i < availablePrintersView.columns; i++) {
                            if (i !== modelData) {
                                columnsHeader.children[i].sortButton.mode = available_printers_model.SORT_NONE;
                            }
                        }
                        toggle();
                    }
                }
                //Bottom header separator
                Rectangle {
                    width: parent.width
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.margins: 0
                    height: 2
                    color: styleUtil.getTableSeparatorColor()
                }
                //Drag handle
                Rectangle {
                    id: dragRectangle
                    anchors.right: parent.right
                    anchors.top: parent.top
                    anchors.margins: 0
                    width: 2
                    height: availablePrintersView.contentHeight - availablePrintersView.contentY
                    color: styleUtil.getTableSeparatorColor()
                    MouseArea {
                        id: dragArea
                        property real dragStart: 0
                        anchors.fill: parent
                        cursorShape: Qt.SizeHorCursor

                        drag.target: parent
                        drag.axis: Drag.XAxis

                        hoverEnabled: true
                        onMouseXChanged: {
                            if(drag.active){
                                var label = headerRepeater.itemAt(index);
                                var labelWidth = label.width + mouseX;
                                var columnWidth = columnwidths[index] + mouseX;
                                if(label.width < 50 || columnWidth < 50) {
                                    label.width = 50
                                    columnwidths[index] = 50;
                                } else {
                                    label.width = labelWidth;
                                    columnwidths[index] = columnWidth;
                                }
                                availablePrintersView.forceLayout();
                            }
                        }

                        //If drag finished: Store column widths
                        drag.onActiveChanged: function () {
                            if (!drag.active) {
                                available_printers_model.storeColumnWidth(index, columnWidthProvider(index));
                            }
                        }
                    }
                }
            }
        }

    }

    //Printer info popup
    Dialog {
        id: printerInfoPopup
        x: 100
        y: 100
        width: 450
        height: 375
        modal: false
        focus: true
        closePolicy: Popup.CloseOnEscape | Popup.CloseOnPressOutsideParent
        standardButtons: Dialog.Ok

        Component.onCompleted: {
            standardButton(Dialog.Ok).text = qsTr("Ok")
        }

        contentItem: ScrollView {
            clip: true
            anchors.left: parent.left
            anchors.leftMargin: 25
            anchors.top: parent.top
            anchors.topMargin: 25
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 50
            width: 300
            height: 450

            PrinterInfo {
                id: printerInfo
                width: parent.width
                height: parent.height
            }
        }
    }

    //Context menu after right click on printer
    Menu {
        id: printerContextMenu
        MenuItem {
            text: qsTr("Install")
            onTriggered: {
                printerInstallDialog.open()
            }
        }
        MenuItem {
            text: qsTr("Info")
            onTriggered: {
                printerInfoPopup.open()
                printerInfo.initData()
            }
        }
    }

    //Cell item delegate
    delegate: Rectangle {
        color: available_printers_model.isCellSelected(row, column) === true ?
                   styleUtil.getAccentColor(): styleUtil.getBackgroundColor();
        //Image used if cell holds an icon
        Image {
            id: cellImage
            visible: false
            property var role: available_printers_model.getColumnRole(column)
            width: 30
            height: 30
            anchors.margins: 10
            Component.onCompleted: init()

            ToolTip.visible: visible && cellMouseArea.containsMouse
            ToolTip.text: visible ? availablePrintersView.iconColumns[this.role][display].tooltip: ""

            //Signals to refresh icons
            Connections {
                target: cellMouseArea
                function onClicked() {
                   cellImage.init();
                }
            }
            Connections {
                target: settingsDialog
                function onAccepted () {
                    cellImage.init();
                }
            }

            //Initialise icon
            function init() {
                var role = available_printers_model.getColumnRole(column);
                this.visible = availablePrintersView.iconColumns[this.role] !== undefined
                if (this.visible) {
                    var icon = availablePrintersView.iconColumns[this.role][display].icon;
                    this.source = styleUtil.getIconPath(icon);
                    this.anchors.centerIn = parent;
                }
            }
        }

        //Text used if cell holds text
        Text {
            text: display
            property var role: available_printers_model.getColumnRole(column)
            visible: availablePrintersView.iconColumns[role] === undefined
            anchors.fill: parent
            anchors.margins: 10
            color: styleUtil.getFontColor()
            font.pixelSize: 15
            verticalAlignment: Text.AlignVCenter
            elide: Text.ElideRight
            ToolTip.visible: visible && cellMouseArea.containsMouse && truncated
            ToolTip.text: text
        }

        MouseArea {
            id: cellMouseArea
            anchors.fill: parent
            acceptedButtons: Qt.LeftButton | Qt.RightButton
            hoverEnabled: true
            onClicked: {
                available_printers_model.handleItemClicked(row, column)
                if (mouse.button === Qt.RightButton) {
                    printerContextMenu.popup()
                }
            }
        }
    }


    Rectangle {
        z: 3
        color: styleUtil.getTableHeaderColor()
        y: availablePrintersView.contentY
        x: availablePrintersView.contentX
        width: availablePrintersView.leftMargin
        height: availablePrintersView.topMargin
    }
}
