/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
import QtQuick 2.2
import QtQuick.Controls 2.2
import QtQuick.Dialogs 1.0 as Dialogs

Dialog {
    height: 300
    width: 600
    standardButtons: Dialog.Apply | Dialog.Cancel

    modal: false
    onApplied: {
        installResultText.text = ""
        installResultReason.text =""
        try {
            var result = available_printers_model.addSelectedPrinter(driverFileText.text);
            installResultText.text = result === true? qsTr("Printer added successfully"): qsTr("Please select a printer");
        } catch (e) {
            installResultText.text = qsTr("Could not add printer:\n")
            installResultReason.text = e.message.trim();
        }
        installResultDialog.open()
        close()
    }

    Component.onCompleted: {
        standardButton(Dialog.Apply).text = qsTr("Apply")
        standardButton(Dialog.Cancel).text = qsTr("Cancel")

        reset()
    }

    contentItem: Pane {
        anchors.fill: parent
        anchors.topMargin: 10
        anchors.leftMargin: 10
        anchors.rightMargin: 10
        background: Rectangle {
            color: "transparent"
        }
        Label {
            id: installDialogLabel
            anchors.top: parent.top
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            font.bold: true
        }
        Label {
            id: installCmdTextLabel
            anchors.top: installDialogLabel.bottom
            anchors.topMargin: 15
            anchors.left: parent.left
            anchors.leftMargin: 5
            font.bold: true
            text: qsTr("Command")
        }

        TextField {
            id: installCmdText
            width: 400
            readOnly: true
            selectByMouse: true
            anchors.top: installCmdTextLabel.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5

        }
        Button {
            id: installCmdCopyButton
            text: qsTr("Copy")
            anchors.top: installCmdTextLabel.bottom
            anchors.topMargin: 5
            anchors.left: installCmdText.right
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            flat: true
            onClicked: {
                //Copy the cmd to clipboard using the invisible TextEdit
                installDialogClipboard.selectAll();
                installDialogClipboard.copy();
            }
        }
        Label {
            id: driverFileLabel
            anchors.top: installCmdText.bottom
            anchors.topMargin: 15
            anchors.left: parent.left
            anchors.leftMargin: 5
            text: qsTr("Driver")
            font.bold: true
        }
        TextField {
            id: driverFileText
            width: 400
            readOnly: true
            selectByMouse: true
            anchors.top: driverFileLabel.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5

        }
        Button {
            id: searchDriverFileButton
            text: qsTr("Search...")
            anchors.top: driverFileLabel.bottom
            anchors.topMargin: 5
            anchors.left: driverFileText.right
            anchors.leftMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            flat: true
            onClicked: {
                fileDialog.open()
            }
        }
        Row {
            id: driverWarnRow
            visible: !controller.isDriverLoaded()
            anchors.left: parent.left
            anchors.top: driverFileText.bottom
            anchors.topMargin: 15
            Image {
                id: driverWarnIcon
                source: styleUtil.getIconPath("warning.svg")
                height: 15
                width: 15
                anchors.margins: 10
            }
            Text {
                text: qsTr("Default printer driver not installed. Open settings to load driver or choose a custom driver.")
                anchors.leftMargin: 15
                wrapMode: Text.WordWrap
                clip: false
                font.pixelSize: 12
                width: printerInstallDialog.width - 50

                MouseArea {
                    anchors.fill: parent
                    cursorShape: "PointingHandCursor"
                    onClicked: {
                        settingsDialog.open();
                    }
                }
            }
        }
    }

    //Invisble TextEdit used to copy the cmd to clipboard
    TextEdit {
        id: installDialogClipboard
        text: installCmdText.text
        visible: false
    }

    function reset() {
        installDialogLabel.text = "";
        installCmdText.text = "";
        driverFileText.text = "";
    }

    function update() {
        driverWarnRow.visible = !controller.isDriverLoaded()
        if (controller.isDriverLoaded()) {
            var file = controller.getDriverPath();
            driverFileText.text = file;
            installCmdText.text = available_printers_model.getSelectedInstallCmd(file);
        }
    }

    onOpened: {
        driverWarnRow.visible = !controller.isDriverLoaded()
        var queue = available_printers_model.getSelectedQueueName();
        installDialogLabel.text = qsTr("Installing printer %1").arg(queue)
        var driver = available_printers_model.getSelectedDriver();
        driverFileText.placeholderText = driver;
        if (controller.isDriverLoaded()) {
            driverFileText.text = controller.getDriverPath();
        }
        var cmd = available_printers_model.getSelectedInstallCmd(driverFileText.text);
        installCmdText.text = cmd;
    }

    Dialogs.FileDialog {
        id: fileDialog
        height: 450
        width: 600
        title: qsTr("Please choose a ppd file")
        folder: shortcuts.home
        selectExisting: true
        selectMultiple: false
        selectFolder: false
        nameFilters: [ qsTr("PPD files (*.ppd)")]

        onAccepted: {
            var file = fileDialog.fileUrl.toString();
            file = file.substring(7);
            driverFileText.text = file;
            installCmdText.text = available_printers_model.getSelectedInstallCmd(file);
        }
    }

    onClosed: reset();
}
