/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Dialogs 1.1
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12

Dialog {
    x: 100
    y: 100
    width: 500
    height: 275
    standardButtons: Dialog.Ok | Dialog.Cancel

    enum Mode {
        PRINTER_LIST,
        DRIVER,
        ALL
    }

    property var mode: SMBDialog.Mode.PRINTER_LIST

    Component.onCompleted: {
        standardButton(Dialog.Ok).text = qsTr("Ok")
        standardButton(Dialog.Cancel).text = qsTr("Cancel")
    }

    onAccepted: {
        submit();
    }

    function refreshLastDriverRefresh() {
        var now = new Date();
        controller.setConfigOption(controller.CFG_NAME_DRIVER_LAST_REFRESH, now);
        controller.saveConfig();
        settingsDialog.init();
    }

    function getPrinterListErrMsg(e) {
        console.log(e);
        if (e.message.includes("NT_STATUS_ACCESS_DENIED")
                || e.message.includes("NT_STATUS_LOGON_FAILURE")) {
            return qsTr("Please check credentials and try again")
        } else if (e.message.includes("NT_STATUS_UNSUCCESSFUL")) {
            return qsTr("The connection to the host could not be established.\nPlease check connection and domain settings and try again.")
        } else if (e.message.includes("NT_STATUS_OBJECT_NAME_NOT_FOUND")) {
            return qsTr("The printer list could not be found on the remote location.\nPlease check settings and try again.")
        } else if (e.message.includes("NT_STATUS_BAD_NETWORK_NAME")) {
            return qsTr("Could not access the SMB Share.\nPlease check connection and domain seetings and ensure you have sufficient permission to access the given directory.")
        } else if (e.message.includes("Could not start smb process")) {
            return qsTr("Could not start smb process")
        } else {
            return  qsTr("Unkown error. Please ensure the connection settings are correct.");
        }
    }

    function getDriverListErrMsg(e) {
        console.log(e);
        if (e.message.includes("NT_STATUS_ACCESS_DENIED")
                || e.message.includes("NT_STATUS_LOGON_FAILURE")) {
            return qsTr("Please check credentials and try again")
        } else if (e.message.includes("NT_STATUS_UNSUCCESSFUL")) {
            return  qsTr("The connection to the host could not be established.\nPlease check connection and domain settings and try again.")
        } else if (e.message.includes("NT_STATUS_OBJECT_NAME_NOT_FOUND")) {
            return  qsTr("The driver could not be found on the remote location.\nPlease check settings and try again.")
        } else if (e.message.includes("NT_STATUS_BAD_NETWORK_NAME")) {
            return  qsTr("Could not access the SMB Share.\nPlease check connection and domain seetings and ensure you have sufficient permission to access the given directory.")
        } else if (e.message.includes("Could not start smb process")) {
            return  qsTr("Could not start smb process")
        } else {
            return  qsTr("Unkown error. Please ensure the connection settings are correct.");
        }
    }

    function submit() {
        try {
            switch (mode) {
            case SMBDialog.Mode.PRINTER_LIST:
                try {
                    available_printers_model.readRemotePrinterListAuthenticated(
                            smbUser.text.toString(), smbPassword.text.toString(), smbDomain.text);
                } catch (e) {
                    smbErrorText.text = getPrinterListErrMsg(e);
                    smbErrorDialog.open();
                }
                break;
            case SMBDialog.Mode.DRIVER:
                try {
                    available_printers_model.readRemoteDriverFileAuthenticated(
                                smbUser.text.toString(), smbPassword.text.toString(), smbDomain.text);
                    refreshLastDriverRefresh();
                } catch (e) {
                    smbErrorText.text = getDriverListErrMsg(e);
                    smbErrorDialog.open();
                }

                break;
            case SMBDialog.Mode.ALL:
                smbErrorText.text = "";
                try {
                    available_printers_model.readRemotePrinterListAuthenticated(
                            smbUser.text.toString(), smbPassword.text.toString(), smbDomain.text);
                } catch (e) {
                    smbErrorText.text += qsTr("Printer list: %1").arg("<p>" + getPrinterListErrMsg(e) + "</p>");
                }
                try {
                    available_printers_model.readRemoteDriverFileAuthenticated(
                                smbUser.text.toString(), smbPassword.text.toString(), smbDomain.text);
                    refreshLastDriverRefresh();
                } catch (e) {
                    smbErrorText.text += qsTr("Driver: %1").arg("<p>" + getDriverListErrMsg(e) + "</p>");
                }
                if (smbErrorText.text !== "") {
                    smbErrorDialog.open();
                }

                break;
            }
            smbUser.clear();
            smbPassword.clear();
            close();
        } catch (e) {
            console.log(e);
            if (e.message.includes("NT_STATUS_ACCESS_DENIED")
                    || e.message.includes("NT_STATUS_LOGON_FAILURE")) {
                smbErrorText.text =  qsTr("Please check credentials and try again")
            } else if (e.message.includes("NT_STATUS_UNSUCCESSFUL")) {
                smbErrorText.text = qsTr("The connection to the host could not be established.\nPlease check connection and domain settings and try again.")
            } else if (e.message.includes("NT_STATUS_OBJECT_NAME_NOT_FOUND")) {
                smbErrorText.text = qsTr("The printer list could not be found on the remote location.\nPlease check settings and try again.")
            } else if (e.message.includes("NT_STATUS_BAD_NETWORK_NAME")) {
                smbErrorText.text = qsTr("Could not access the SMB Share.\nPlease check connection and domain seetings and ensure you have sufficient permission to access the given directory.")
            } else if (e.message.includes("Could not start smb process")) {
                smbErrorText.text = qsTr("Could not start smb process")
            } else {
                smbErrorText.text = "";
            }

            smbErrorDialog.open();
        }
    }

    contentItem: Pane {
        anchors.fill: parent
        anchors.topMargin: 10
        anchors.leftMargin: 10
        anchors.rightMargin: 10
        background: Rectangle {
            color: "transparent"
        }

        Label {
                id: formHeader
                anchors.top: parent.top
                anchors.topMargin: 10
                anchors.left: parent.left
                text: {
                    switch (mode) {
                    case SMBDialog.Mode.PRINTER_LIST:
                        qsTr("Enter SMB credentials to access printer list on %1")
                            .arg("<i><p>" + controller.getPrinterListSmbUri(false) + "</p></i>");
                        break;
                    case SMBDialog.Mode.DRIVER:
                        qsTr("Enter SMB credentials to access printer driver on %1")
                            .arg("<i><p>" + controller.getDriverSmbUri(false) + "</p></i>")
                        break;
                    case SMBDialog.Mode.ALL:
                        qsTr("Enter SMB credentials to access printer list and driver on %1")
                            .arg("<i><p>" + controller.getDriverSmbUri(false) + "</p></i>")
                        break;
                    }
                }
                font.pixelSize: 12
                font.bold: true
                width: parent.width - 20
                wrapMode: Text.WordWrap
        }
        Label {
            id: usernameLabel
            text: qsTr("Username:")
            font.bold: true
            anchors.topMargin: 15
            anchors.top: formHeader.bottom
        }
        Label {
            id: passwordLabel
            text: qsTr("Password:")
            font.bold: true
            anchors.margins: 25
            anchors.top: usernameLabel.bottom
        }
        Label {
            id: domainLabel
            text: qsTr("Domain:")
            font.bold: true
            anchors.margins: 25
            anchors.top: passwordLabel.bottom
        }
        TextField {
            id: smbUser
            placeholderText: qsTr("Enter name")
            width: 250
            focus: true
            anchors.top: usernameLabel.top
            anchors.topMargin: -10
            anchors.left: usernameLabel.right
            anchors.leftMargin: 10
        }

        TextField {
            id: smbPassword
            placeholderText: qsTr("Enter password")
            width: 250
            echoMode: TextInput.Password
            anchors.top: passwordLabel.top
            anchors.topMargin: -10
            anchors.left: usernameLabel.right
            anchors.leftMargin: 10
            Keys.onReturnPressed: {
                submit();
            }
        }

        TextField {
            id: smbDomain
            placeholderText: qsTr("Domain")
            width: 250
            anchors.top: domainLabel.top
            anchors.topMargin: -10
            anchors.left: usernameLabel.right
            anchors.leftMargin: 10
            Keys.onReturnPressed: {
                submit();
            }
        }
    }

    Dialog {
        id: smbErrorDialog
        x:printerInstallDialog.x + 25
        y:printerInstallDialog.y + 25
        height: 200
        width: 450
        modal: false
        standardButtons: Dialog.Ok
        contentItem: Pane {
            anchors.fill: parent
            anchors.topMargin: 10
            anchors.rightMargin: 10
            anchors.leftMargin: 10
            background: Rectangle {
                color: "transparent"
            }
            Label {
                id: smbErrorHeader
                text:{
                    switch (mode) {
                    case SMBDialog.Mode.PRINTER_LIST:
                        qsTr("Error reading the remote printer list");
                        break;
                    case SMBDialog.Mode.DRIVER:
                        qsTr("Error loading the remote printer driver");
                        break;
                    case SMBDialog.Mode.ALL:
                        qsTr("Error loading the remote printer list and driver")
                        break;
                    }
                }
                font.bold: true
                anchors.margins: 5
            }
            Label {
                id: smbErrorText
                anchors.top: smbErrorHeader.bottom
                anchors.margins: 10
                width: parent.width
                text: ""
                wrapMode: Text.WordWrap
            }
        }
    }
}
