/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.3

//Settings dialog
Dialog {
    closePolicy: Popup.CloseOnEscape
    standardButtons: Dialog.Ok | Dialog.Cancel

    x: 100
    y: 100
    width: mainWindow.width - 100 < 600 ? mainWindow.width - 100: 600;
    height: mainWindow.height - 100 < 700 ? mainWindow.height - 100: 700;

    title: qsTr("Settings")

    Component.onCompleted: {
        standardButton(Dialog.Ok).text = qsTr("Ok")
        standardButton(Dialog.Cancel).text = qsTr("Cancel")
    }

    function init() {
        settingsGrid.init();
    }

    onOpened: {
        settingsGrid.init();
    }

    contentItem: ScrollView {
        anchors.fill: parent
        anchors.topMargin: 50
        anchors.bottomMargin: 50
        anchors.leftMargin: 20
        anchors.rightMargin: 20
        id: settingsGrid
        clip: true
        contentHeight: uiBox.height + smbBox.height + driverBox.height + 50

        function init () {
            controller.reloadCfg();
            themeSelection.setValue(controller.getConfigOption(controller.CFG_NAME_UI_THEME));
            languageSelection.setValue(controller.getConfigOption(controller.CFG_NAME_UI_LANG));
            smbHostInput.text = controller.getConfigOption(controller.CFG_NAME_SMB_HOST);
            smbShareInput.text = controller.getConfigOption(controller.CFG_NAME_SMB_SHARE);
            smbFileInput.text = controller.getConfigOption(controller.CFG_NAME_SMB_FILE);
            smbUri.text = controller.getPrinterListSmbUri(true);
            driverFileInput.text = controller.getConfigOption(controller.CFG_NAME_DRIVER_FILE)
            driverHostInput.text = controller.getConfigOption(controller.CFG_NAME_DRIVER_HOST);
            driverShareInput.text = controller.getConfigOption(controller.CFG_NAME_DRIVER_SHARE);
            driverUri.text = controller.getDriverSmbUri(true);
            var timestamp = controller.getConfigOption(controller.CFG_NAME_DRIVER_LAST_REFRESH);
            if (timestamp === "") {
                timestamp = qsTr("never")
            }

            driverLastRefreshLabel.text = qsTr("Last refresh: %1").arg(timestamp);
        }

        function refreshSmbUri() {
            smbUri.text = controller.getSmbUri(smbHostInput.text, smbShareInput.text, smbFileInput.text);
        }

        function refreshDriverUri() {
            driverUri.text = controller.getSmbUri(driverHostInput.text, driverShareInput.text, driverFileInput.text);
        }

        Component.onCompleted: {
            settingsGrid.init();
        }

        GroupBox {
            id: uiBox
            title: qsTr("UI")
            anchors.top: parent.top
            anchors.margins: 10
            Grid {
                columns: 2
                spacing: 10
                verticalItemAlignment: Grid.AlignVCenter
                horizontalItemAlignment: Grid.AlignLeft
                Label {
                    id: themeLabel
                    text: qsTr("Theme")
                    font.bold: true
                }
                ComboBox {
                    id: themeSelection
                    editable: true
                    textRole: "text"
                    model: [
                        { value: controller.CFG_VALUE_THEME_LIGHT, text: qsTr("Light") },
                        { value: controller.CFG_VALUE_THEME_DARK, text: qsTr("Dark") }
                    ]
                    function getValue() {
                        var index = themeSelection.currentIndex;
                        return themeSelection.model[currentIndex].value;
                    }
                    function setValue(newValue) {
                        for (var i = 0; i < model.length; i++) {
                            var val = model[i].value;
                            if (val === newValue) {
                                themeSelection.currentIndex = i;
                                return;
                            }
                        }
                    }
                }
                Label {
                    id: languageLabel
                    text: qsTr("Language")
                    font.bold: true
                }
                ComboBox {
                    id: languageSelection
                    editable: true
                    textRole: "text"
                    model: [
                        { value: "auto", text: qsTr("Auto")},
                        { value: "en_US", text: qsTr("English") },
                        { value: "de_CH", text: qsTr("German") }
                    ]
                    function getValue() {
                        var index = languageSelection.currentIndex;
                        return languageSelection.model[currentIndex].value;
                    }
                    function setValue(newValue) {
                        for (var i = 0; i < model.length; i++) {
                            var val = model[i].value;
                            if (val === newValue) {
                                languageSelection.currentIndex = i;
                                return;
                            }
                        }
                    }
                }
            }
        }
        GroupBox {
            id: smbBox
            anchors.top: uiBox.bottom
            anchors.margins: 10
            title: qsTr("Printer List")
            Grid {
                columns: 2
                spacing: 10
                verticalItemAlignment: Grid.AlignVCenter
                horizontalItemAlignment: Grid.AlignLeft

                Label {
                    id: smbHostLabel
                    text: qsTr("SMB Host")
                    font.bold: true
                }
                TextField {
                    id: smbHostInput
                    width: 250
                    selectByMouse: true
                    onTextChanged: settingsGrid.refreshSmbUri()
                }
                Label {
                    id: smbShareLabel
                    text: qsTr("SMB Share")
                    font.bold: true
                }
                TextField {
                    id: smbShareInput
                    width: 250
                    selectByMouse: true
                    onTextChanged: settingsGrid.refreshSmbUri()
                }
                Label {
                    id: smbFileLabel
                    text: qsTr("Printer Definitions File")
                    font.bold: true
                }
                TextField {
                    id: smbFileInput
                    width: 250
                    selectByMouse: true
                    onTextChanged: settingsGrid.refreshSmbUri()
                }
                Label {
                    id: smbUriLabel
                    text: qsTr("SMB Uri")
                    font.bold: true
                }
                Text {
                    id: smbUri
                    width: 250
                    font.pixelSize: 14
                    elide: Text.ElideRight
                    ToolTip.visible: smbUriMouseArea.containsMouse && smbUri.truncated
                    ToolTip.text: smbUri.text
                    color: styleUtil.getFontColor()

                    MouseArea {
                        id: smbUriMouseArea
                        anchors.fill: parent
                        anchors.margins: -10
                        hoverEnabled: true
                    }
                }
            }
        }
        GroupBox {
            id: driverBox
            anchors.top: smbBox.bottom
            anchors.margins: 10
            title: qsTr("Default Driver")
            ColumnLayout {
                Grid {
                    id: driverGrid
                    columns: 2
                    spacing: 10
                    verticalItemAlignment: Grid.AlignVCenter
                    horizontalItemAlignment: Grid.AlignLeft
                    Label {
                        id: driverHostLabel
                        text: qsTr("SMB Host")
                        font.bold: true
                    }
                    TextField {
                        id: driverHostInput
                        width: 250
                        selectByMouse: true
                        onTextChanged: settingsGrid.refreshDriverUri()
                    }
                    Label {
                        id: driverShareLabel
                        text: qsTr("SMB Share")
                        font.bold: true
                    }
                    TextField {
                        id: driverShareInput
                        width: 250
                        selectByMouse: true
                        onTextChanged: settingsGrid.refreshDriverUri()
                    }
                    Label {
                        id: driverFileLabel
                        text: qsTr("Driver File")
                        font.bold: true
                    }
                    TextField {
                        id: driverFileInput
                        width: 250
                        selectByMouse: true
                        onTextChanged: settingsGrid.refreshDriverUri()
                    }
                    Label {
                        id: driverUriLabel
                        text: qsTr("SMB Uri")
                        font.bold: true
                    }
                    Text {
                        id: driverUri
                        width: 250
                        font.pixelSize: 14
                        elide: Text.ElideRight
                        ToolTip.visible: driverUriMouseArea.containsMouse && driverUri.truncated
                        ToolTip.text: driverUri.text
                        color: styleUtil.getFontColor()

                        MouseArea {
                            id: driverUriMouseArea
                            anchors.fill: parent
                            anchors.margins: -10
                            hoverEnabled: true
                        }
                    }
                }
                RowLayout {
                    Layout.alignment: Layout.TopLeft
                    Label {
                        id: driverLastRefreshLabel
                        text: ""
                    }
                    Pane {
                        background: Rectangle {
                            color: "transparent"
                        }
                        Layout.alignment: Layout.Left
                        Button {
                            id: driverReloadButton
                            anchors.fill: parent
                            flat: true
                            Image {
                                source: styleUtil.getIconPath("refresh.svg")
                                height: 25
                                width: 25
                                anchors.centerIn: parent
                            }

                            ToolTip.visible: hovered
                            ToolTip.text: qsTr("Reload driver")
                            onClicked: {
                                //Set driver config and update
                                controller.setConfigOption(controller.CFG_NAME_DRIVER_FILE, driverFileInput.text);
                                controller.setConfigOption(controller.CFG_NAME_DRIVER_HOST, driverHostInput.text);
                                controller.setConfigOption(controller.CFG_NAME_DRIVER_SHARE, driverShareInput.text);
                                controller.saveConfig();
                                smbDialog.mode = SMBDialog.Mode.DRIVER;
                                smbDialog.open();
                            }
                        }
                    }
                }
            }
        }
    }

    onAccepted: {
        controller.setConfigOption(controller.CFG_NAME_UI_THEME, themeSelection.getValue());
        controller.setConfigOption(controller.CFG_NAME_SMB_HOST, smbHostInput.text);
        controller.setConfigOption(controller.CFG_NAME_SMB_SHARE, smbShareInput.text);
        controller.setConfigOption(controller.CFG_NAME_SMB_FILE, smbFileInput.text);
        controller.setConfigOption(controller.CFG_NAME_UI_LANG, languageSelection.getValue());
        controller.setConfigOption(controller.CFG_NAME_DRIVER_FILE, driverFileInput.text);
        controller.setConfigOption(controller.CFG_NAME_DRIVER_HOST, driverHostInput.text);
        controller.setConfigOption(controller.CFG_NAME_DRIVER_SHARE, driverShareInput.text);

        if (controller.saveConfig()) {
            styleUtil.switchTheme(themeSelection.getValue());
            translator.switchLanguage(languageSelection.getValue());

            printerInstallDialog.update();
        } else {
            console.log("Cfg not saved")
        }
    }
    onRejected: {
        settingsGrid.init();
    }
}
