/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
#include "configutility.h"

#include "stringutils.h"

#include <cstdlib>
#include <fstream>
#include <regex>
#include <sstream>
#include <stdexcept>
#include <syscall.h>

#include <iostream>

ConfigUtility* ConfigUtility::instance = nullptr;

ConfigUtility::ConfigUtility() {
    //Set cfg file path to user config dir
    std::stringstream cfgStream;

    std::string home = std::getenv("HOME");
    cfgStream << home;
    if (home.at(home.length() - 1) != '/') {
        cfgStream << "/";
    }
    cfgStream << cfgFilePathPostfix;
    cfgFilePath = cfgStream.str();
    fullPath = cfgFilePath + "/" + cfgFileName;

    globalFullPath = globalCfgPath + "/" + globalCfgFileName;

    options = new std::map<std::string, std::string>;
    initCfg();
    //If cfg file already exists: read it,
    //else create default cfg
    if (checkUserCfgFile()) {
        readCfgFile();
    } else {
        createCfgDir();
        saveUserCfg();
    }
}

bool ConfigUtility::checkDriverFile() {
    std::ifstream f(getDriverPath().c_str());
    bool good = f.good();

    if (!good) {
        setCfgOPt(CFG_NAME_DRIVER_LAST_REFRESH, "");
        saveUserCfg();
    }
    return good;
}

ConfigUtility* ConfigUtility::getInstance() {
    if (instance == nullptr) {
        instance = new ConfigUtility();
    }
    return instance;
}

void ConfigUtility::reloadCfg() {
    readCfgFile();
}

std::string ConfigUtility::getCfgOpt(std::string key) {
    std::string value;
    try {
        value = options->at(key);
    } catch (std::out_of_range) {
        std::stringstream errorstream;
        errorstream << "Key " << key << " not found!";
        throw new ConfigException(errorstream.str());
    }
    return value;
}

void ConfigUtility::setCfgOPt(std::string key, std::string value) {
    if (options->find(key) == options->end()) {
        std::stringstream errorstream;
        errorstream << "Key " << key << " not found!";
        throw new ConfigException(errorstream.str());
    }
    options->at(key) = value;
}

std::string ConfigUtility::getCfgPath() {
    return cfgFilePath;
}

bool ConfigUtility::checkGlobalCfgFile() {
    std::ifstream f(globalFullPath.c_str());
    return f.good();
}

bool ConfigUtility::checkUserCfgFile() {
    std::ifstream f(fullPath.c_str());
    return f.good();
}

void ConfigUtility::createCfgDir() {
    std::string mkdir = "mkdir -p " + cfgFilePath;
    int mkdirResult = std::system(mkdir.c_str());
    if (mkdirResult != 0) {
        throw IOException("Could not generate config directories");
    }
}

std::string ConfigUtility::getDriverPath() {
    std::string destFile = getCfgPath();
    if (destFile.at(destFile.length() - 1) != '/') {
        destFile += "/";
    }
    //Append driver filename without any parent directories
    std::string fileName = getCfgOpt(CFG_NAME_DRIVER_FILE);
    size_t lastSlash = fileName.find_last_of("/");

    if (lastSlash == std::string::npos) {
        destFile += fileName;
    } else {
         destFile += fileName.substr(lastSlash + 1, fileName.length() - 1);
    }
    return destFile;
}

void ConfigUtility::initCfg() {
    options->clear();
    options->insert(std::pair<std::string, std::string>(CFG_NAME_SMB_HOST, "smb_host"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_SMB_SHARE, "smb_share"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_SMB_FILE, "smb_file"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_UI_LANG, "en_US"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_UI_THEME, "light"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_UI_COLUMN_WIDTHS, ""));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_DRIVER_FILE, "driver_file"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_DRIVER_HOST, "driver_host"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_DRIVER_SHARE, "driver_share"));
    options->insert(std::pair<std::string, std::string>(CFG_NAME_DRIVER_LAST_REFRESH, ""));

    if (checkGlobalCfgFile()) {
        readCfgFile(globalFullPath);
    } else {
        std::cout << "Can not read global config file at: " << globalFullPath << std::endl;
    }
}

void ConfigUtility::readCfgFile() {
    readCfgFile(fullPath);
}

void ConfigUtility::readCfgFile(std::string path) {
    std::map<std::string, std::string> parsedOptions;
    std::ifstream cfgFile(path.c_str());

    //Parse config file
    std::string line;
    while (std::getline(cfgFile, line)) {
        std::string begin = line.substr(0, 1);
        line = StringUtils::trim(line);
        //Skip empty lines
        if (line.length() == 0) {
            continue;
        }
        //Skip comments
        if (std::regex_match(begin, CFG_FILE_COMMENT_REGEX)) {
            continue;
        }
        size_t delimiterPos = line.find(CFG_FILE_DELIMITER);
        if (delimiterPos == std::string::npos) {
            std::cout << "Unable to parse line: " << line << std::endl;
            throw new ConfigParserException("Unable to parse line: " + line);
        }
        std::string key = StringUtils::trim(line.substr(0, delimiterPos));
        std::string value = StringUtils::trim(line.substr(delimiterPos + 1, line.length() -1));
        parsedOptions.insert(std::pair<std::string, std::string>(key, value));
    }
    //Insert parsed config into options map
    for (std::map<std::string, std::string>::iterator it = parsedOptions.begin(); it != parsedOptions.end(); ++it) {
        try {
            std::string key = it->first;
            std::string &optionsValue = options->at(key);
            optionsValue = it->second;
        } catch (std::out_of_range) {
            continue;
        }
    }
    cfgFile.close();
}

void ConfigUtility::saveUserCfg() {
    std::ofstream cfgFile(fullPath, std::ofstream::trunc);

    for (std::map<std::string, std::string>::iterator it = options->begin(); it != options->end(); ++it) {
        cfgFile << it->first << " = " << it->second << std::endl;
    }

    cfgFile.close();
}


