/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "configutility.h"
#include "cupsutility.h"
#include "smbutility.h"

#include <QObject>

/**
 * @brief Controller class providing common functions callable from qml
 */
class Controller : public QObject {
    Q_OBJECT
public:
    explicit Controller(QObject *parent = nullptr);

    /**
     * @brief Reload config
     */
    Q_INVOKABLE void reloadCfg();

    /**
     * @brief Save current config
     * @return True if successful
     */
    Q_INVOKABLE bool saveConfig();

    /**
     * @brief Set a config option by key
     * @param key Config option key
     * @param value New value
     * @return True if successful
     */
    Q_INVOKABLE bool setConfigOption(QString key, QString value);

    /**
     * @brief Get config option by key
     * @param key Key to look up
     * @return Option as string
     */
    Q_INVOKABLE QString getConfigOption(QString key);

    /**
     * @brief Get the build time as QString
     * @return Build time as QString
     */
    Q_INVOKABLE QString getBuildTime();

    /**
     * @brief Get the hg commit hash as QString
     * @return Commit hash as QString
     */
    Q_INVOKABLE QString getCommitHash();

    /**
     * @brief Get the local default driver path
     * @return Path as QString
     */
    Q_INVOKABLE QString getDriverPath();

    /**
     * @brief Get the smb uri using the current settings
     * @param includeFile Set if filename should be included
     * @return Uri as QString
     */
    Q_INVOKABLE QString getPrinterListSmbUri(bool includeFile);

    /**
     * @brief Get the driver file smb uri
     * @param includeFile Set if filename should be included
     * @return Uri as QString
     */
    Q_INVOKABLE QString getDriverSmbUri(bool includeFile);

    /**
     * @brief Get the smb uri using the given values
     * @param host Host
     * @param share Share
     * @param QString file
     * @return Uri as QString
     */
    Q_INVOKABLE QString getSmbUri(QString host, QString share, QString file);

    /**
     * @brief Get application version as QString
     *
     * Format: MAJOR.MINOR.PATCH
     * @return Version as QString
     */
    Q_INVOKABLE QString getVersion();

    /**
     * @brief Check if default driver is loaded
     * @return True if loaded
     */
    Q_INVOKABLE bool isDriverLoaded();

    /**
     * @brief Check if all neccessary binaries are available
     * Throws an exception using the QJSEngine if failed.
     * @return True if passed, else false
     */
    Q_INVOKABLE bool selfCheck();

    //Expose constants to QML
    //Config names
    Q_PROPERTY(QString CFG_NAME_SMB_HOST READ getCfgNameSmbHost)
    Q_PROPERTY(QString CFG_NAME_SMB_SHARE READ getCfgNameSmbShare)    
    Q_PROPERTY(QString CFG_NAME_SMB_FILE READ getCfgNameSmbFile)    
    Q_PROPERTY(QString CFG_NAME_UI_LANG READ getCfgNameUiLang)
    Q_PROPERTY(QString CFG_NAME_UI_THEME READ getCfgNameUiTheme)
    Q_PROPERTY(QString CFG_NAME_DRIVER_HOST READ getCfgDriverHost)
    Q_PROPERTY(QString CFG_NAME_DRIVER_SHARE READ getCfgDriverShare)
    Q_PROPERTY(QString CFG_NAME_DRIVER_FILE READ getCfgDriverFile)
    Q_PROPERTY(QString CFG_NAME_DRIVER_LAST_REFRESH READ getCfgDriverLastRefresh)

    //Config values
    Q_PROPERTY(QString CFG_VALUE_THEME_LIGHT READ getCfgValueThemeLight CONSTANT);
    Q_PROPERTY(QString CFG_VALUE_THEME_DARK READ getCfgValueThemeDark CONSTANT);
    Q_PROPERTY(QString CFG_VALUE_LANG_AUTO READ getCfgValueLangAuto CONSTANT);

private:
    ConfigUtility* cfg;    
    SMBUtility* smbUtility;
    CupsUtility* cupsUtility;

    QString getCfgNameSmbHost();
    QString getCfgNameSmbShare();    
    QString getCfgNameSmbFile();
    QString getCfgNameUiLang();
    QString getCfgNameUiTheme();    

    QString getCfgDriverHost();
    QString getCfgDriverShare();
    QString getCfgDriverFile();
    QString getCfgDriverLastRefresh();

    QString getCfgValueThemeLight();
    QString getCfgValueThemeDark();
    QString getCfgValueLangAuto();


signals:

};

#endif // CONTROLLER_H
