﻿/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */
#include "smbutility.h"

#include <array>
#include <cstdio>
#include <iostream>
#include <memory>
#include <sstream>
#include <stdexcept>
#include <regex>
#include <sstream>
#include <string>
#include <syscall.h>

#include <QProcess>
#include <QString>
#include <QTemporaryFile>

#include <QDebug>

int SMBUtility::debugLevel = 0;

SMBUtility::SMBUtility() {
    cfg = ConfigUtility::getInstance();

    //Get smbclient version
    QStringList args;
    args.append(CMD_VERSION);
    try {
        std::string versionString = this->exec(CMD, args);
        versionString = versionString.erase(0, 8);
        versionString = versionString.erase(versionString.find_last_of("\n"), 1);
        if (debugLevel == 2) {
            qDebug() << "SmbClient version: " << QString::fromStdString(versionString);
        }
        this->setSmbClientVersion(versionString);
    } catch (SMBException* smbe) {
        qDebug("Could not get smbclient version");
    }
}

std::string SMBUtility::exec(const QString cmd, const QStringList args) {
    QProcess proc;
    proc.start(cmd, args);
    if (!proc.waitForStarted()) {
        throw new SMBException("Could not start smb process");
    }
    proc.waitForFinished();
    bool result = proc.exitCode() == 0;
    QString output(proc.readAllStandardOutput());
    proc.close();
    if (!result) {
        throw new SMBException(output.toStdString());
    } else {
        return output.toStdString();
    }
}

std::string SMBUtility::exec(const std::string cmd) {
    std::array<char, 128> buffer;
    std::string result;

    auto pipe = popen(cmd.c_str(), "r");

    if (!pipe){
        throw std::runtime_error("popen() failed!");
    }

    while (!feof(pipe)) {
        if (fgets(buffer.data(), 128, pipe) != nullptr)
            result += buffer.data();
    }

    auto rc = pclose(pipe);
    if (rc != EXIT_SUCCESS) {
        throw new SMBException(result);
    }
    return result;
}

std::string SMBUtility::getRemoteConfigFile(std::string user, std::string password, std::string domain) {

    QStringList args;
    QTemporaryFile auth;
    std::stringstream outStream;

    if (!auth.open()) {
        throw new SMBException("Could not create temporary auth file");
    }
    auth.setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner);
    outStream << "username = " << user << std::endl
              << "password = " << password << std::endl;
    if (domain.size() > 0) {
        outStream << "domain = " << domain << std::endl;
    }
    if (auth.write(outStream.str().data()) == -1) {
        auth.remove();
        throw new SMBException("Could not write to temporary auth file");
    }
    auth.close();

    if (domain.length() > 0) {
        args << CMD_DOMAIN << QString::fromStdString(domain);
    }
    //Get printer list
    QString getCmd = CMD_SOURCE_PREFIX;
    getCmd.append(QString::fromStdString(cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE)));
    getCmd.append(CMD_DEST_PREFIX);
    getCmd.append(QString::fromStdString(TMP_PRINTER_FILE));
    getCmd.append(CMD_DEST_POSTFIX);
    args << CMD_AUTH_FILE
          << auth.fileName()
          << getEncryptionParam()
          << QString::fromStdString(getPrinterListSmbURI())
          << CMD_SOURCE
          << getCmd;
    if (debugLevel == 2) {
        qDebug() << CMD << " " << args;
    }

    std::string result = exec(CMD, args);
    return TMP_PRINTER_FILE;
}

std::string SMBUtility::getRemoteConfigFileAnonymous() {
    QStringList args;

    QString getCmd = CMD_SOURCE_PREFIX;
    getCmd.append(QString::fromStdString(cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE)));
    getCmd.append(CMD_DEST_PREFIX);
    getCmd.append(QString::fromStdString(TMP_PRINTER_FILE));
    getCmd.append(CMD_DEST_POSTFIX);

    args  << QString::fromStdString(getPrinterListSmbURI())
          << CMD_ANON
          << CMD_SOURCE
          << getCmd;
    if (debugLevel == 2) {
        qDebug() << CMD << " " << args;
    }
    std::string result = exec(CMD, args);
    return TMP_PRINTER_FILE;
}

std::string SMBUtility::getRemoteDriverFile(std::string user, std::string password, std::string domain) {

    QStringList args;
    QTemporaryFile auth;
    std::stringstream outStream;
    std::string destFile;

    destFile = cfg->getDriverPath();
    qDebug() << "Copying driver to " << destFile.c_str();
    if (!auth.open()) {
        throw new SMBException("Could not create temporary auth file");
    }
    auth.setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner);
    outStream << "username = " << user << std::endl
              << "password = " << password << std::endl;
    if (domain.size() > 0) {
        outStream << "domain = " << domain << std::endl;
    }
    if (auth.write(outStream.str().data()) == -1) {
        auth.remove();
        qDebug() << "Writing to auth file failed";
        throw new SMBException("Could not write to temporary auth file");
    }
    auth.close();

    if (domain.length() > 0) {
        args << CMD_DOMAIN << QString::fromStdString(domain);
    }
    //Get driver
    QString getCmd = CMD_SOURCE_PREFIX;
    getCmd.append(QString::fromStdString(cfg->getCfgOpt(cfg->CFG_NAME_DRIVER_FILE)));
    getCmd.append(CMD_DEST_PREFIX);
    getCmd.append(QString::fromStdString(destFile));
    getCmd.append(CMD_DEST_POSTFIX);
    args << CMD_AUTH_FILE
          << auth.fileName()
          << getEncryptionParam()
          << QString::fromStdString(getDriverSmbUri(false))
          << CMD_SOURCE
          << getCmd;
    if (debugLevel == 2) {
        qDebug() << CMD << " " << args;
    }
    std::string result = exec(CMD, args);
    return TMP_PRINTER_FILE;
}

std::string SMBUtility::getPrinterListSmbURI() {
    return getPrinterListSmbURI(false);
}

std::string SMBUtility::getPrinterListSmbURI(bool includeFile) {
    std::string host = cfg->getCfgOpt(cfg->CFG_NAME_SMB_HOST);
    std::string share = cfg->getCfgOpt(cfg->CFG_NAME_SMB_SHARE);
    std::string file = cfg->getCfgOpt(cfg->CFG_NAME_SMB_FILE);
    return getSmbURI(host, share, includeFile? file: "");
}

std::string SMBUtility::getDriverSmbUri(bool includeFile) {
    std::string host = cfg->getCfgOpt(cfg->CFG_NAME_DRIVER_HOST);
    std::string share = cfg->getCfgOpt(cfg->CFG_NAME_DRIVER_SHARE);
    std::string file = cfg->getCfgOpt(cfg->CFG_NAME_DRIVER_FILE);
    return getSmbURI(host, share, includeFile? file: "");
}

std::string SMBUtility::getSmbURI(std::string host, std::string share, std::string file) {
    std::stringstream uri;
    if (host.substr(0, 2).compare("//") != 0) {
        uri  << "//";
    }

    uri << host;
    if (host.find_last_of("/") != host.length() - 1) {
        uri << "/";
    }
    uri << share;
    if (file.length() > 0) {
        uri << "/" << file;
    }
    return uri.str();
}

bool SMBUtility::selfCheck() {
    QDebug debug = qDebug();
    debug << "SmbUtlity self check...";
    QProcess proc;
    proc.start(CMD);
    if (!proc.waitForStarted()) {
        debug << "failed";
        throw new SMBException("Could not find smbclient binary");
    }
    proc.waitForFinished();
    debug << "passed";
    return true;
}

void SMBUtility::setDebugLevel(int debugLevel) {
    SMBUtility::debugLevel = debugLevel;
}

QString SMBUtility::getEncryptionParam() {
    if (this->smbClientVersion >= V4_15) {
        return CMD_CLIENT_PROTECTION;
    } else {
        return CMD_ENCRYPT;
    }
}

void SMBUtility::setSmbClientVersion(std::string versionString)  {
    std::vector<std::string> parts{};
    size_t pos;
    while((pos = versionString.find(".")) != std::string::npos) {
        parts.push_back(versionString.substr(0, pos));
        versionString.erase(0, pos + 1);
    }
    pos = versionString.find("-");
    if (pos == std::string::npos) {
        parts.push_back(versionString);
        parts.push_back("");
    } else {
        parts.push_back(versionString.substr(0, pos));
        parts.push_back(versionString.substr(pos+1, std::string::npos));
    }
    this->smbClientVersion = {
        std::stoi(parts.at(0)), std::stoi(parts.at(1)), std::stoi(parts.at(2)), parts.at(3)
    };
}

bool SmbClientVersion::operator>=(const SmbClientVersion &other) const {
    return major >= other.major
            && minor >= other.minor
            && patch >= other.patch
            && suffix >= other.suffix;
}
