/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */

#ifndef SMBUTILITY_H
#define SMBUTILITY_H

#include <exception>
#include <string>

#include <QString>
#include <QStringList>

#include "configutility.h"

class SMBException: public std::exception {
private:
    std::string message;
public:
    explicit SMBException(const std::string& message) {
        this->message = message;
    };
    const char* what() const noexcept override {
        return message.c_str();
    }
};

struct SmbClientVersion {
    int major;
    int minor;
    int patch;
    std::string suffix;

    bool operator>=(const SmbClientVersion &) const;
};

/**
 * @brief Class providing functions to copy files from a smb share
 */
class SMBUtility {

public:

    SMBUtility();

    /**
     * @brief Executes the given command using QProcess and returns the output
     * @param cmd Command to execute
     * @param args cmd arguments
     * @return Cmd output
     */
    std::string exec(const QString cmd, const QStringList args);

    /**
     * @brief Executes the given command and returns the output
     * @param cmd Command to execute
     * @return
     */
    std::string exec(const std::string cmd);

    /**
     * @brief Read the remote file and copy to local filesystem using the given credentials
     * @param user Username
     * @param password password
     * @param domain SMB Domain
     * @return Local file path
     */
    std::string getRemoteConfigFile(std::string user, std::string password, std::string domain);

    /**
     * @brief Read the remote file and copy to local filesystem without using credentials
     * @return Local file path
     */
    std::string getRemoteConfigFileAnonymous();

    /**
     * @brief Read the remote driver file and copy to local filesystem using the given credentials
     * @param user Username
     * @param password Password
     * @param domain SMB Domain
     * @return Local file path
     */
    std::string getRemoteDriverFile(std::string user, std::string password, std::string domain);

    /**
     * @brief Get the smb uri using the config.
     *
     * The printer definitions filename will not be included in the uri
     * @return Uri as string
     */
    std::string getPrinterListSmbURI();

    /**
     * @brief Get the smb uri using the config
     * @param includeFile Set if printer definitions file should be included
     * @return Uri as string
     */
    std::string getPrinterListSmbURI(bool includeFile);

    /**
     * @brief Get the driver smb uri
     * @param includeFile Set if filename should be included
     * @return Uri as string
     */
    std::string getDriverSmbUri(bool includeFile);

    /**
     * @brief Get uri from given parameters
     * @param host Host
     * @param share Share
     * @param file File
     * @return Uri as string
     */
    std::string getSmbURI(std::string host, std::string share, std::string file);

    /**
     * @brief Check if smbclient binary is available
     * @throws SmbException* If check failed
     * @return True if passed
     */
    bool selfCheck();

    static void setDebugLevel(int debugLevel);

    /**
     * @brief Temp dir to copy remote files to
     */
    const std::string TMP_PRINTER_FILE = "/tmp/printers_config.txt";

private:

    ConfigUtility *cfg;

    const QString CMD = "smbclient";
    const QString CMD_AUTH_FILE = "-A";
    const QString CMD_DOMAIN= "-W";
    const QString CMD_ANON = " --no-pass ";
    const QString CMD_CREDENTIALS ="--user";
    const QString CMD_ENCRYPT = "--encrypt";
    const QString CMD_CLIENT_PROTECTION = "--client-protection=encrypt";
    const QString CMD_SOURCE = "--command";
    const QString CMD_SOURCE_PREFIX = "get \"";
    const QString CMD_DEST_PREFIX = "\" \"";
    const QString CMD_DEST_POSTFIX = "\"";
    const QString CMD_OUTPUT_POSTFIX = " 2>&1";
    const QString CMD_NO_OUTPUT_POSTFIX = "> /dev/null 2>&1";
    const QString CMD_VERSION = "--version";

    const SmbClientVersion V4_15 = {4, 15, 0, ""};

    static int debugLevel;

    SmbClientVersion smbClientVersion;

    QString getEncryptionParam();
    void setSmbClientVersion(std::string versionString);

};

#endif // SMBUTILITY_H
