/*
 * SPDX-License-Identifier: LGPL-3.0-or-later
 * SPDX-FileCopyrightText: 2020 Intevation GmbH <https://intevation.de>
 * Funded by: ETHZ
 */

#ifndef TABLEMODEL_H
#define TABLEMODEL_H

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlEngine>
#include <QAbstractTableModel>

#include <QMouseEvent>

#include <string>

#include "cupsutility.h"
#include "printerparser.h"
#include "printermodel.h"
#include "smbutility.h"

/**
 * @brief Class holding table data and providing functions for filtering and sorting
 */
class TableModel : public QAbstractTableModel {
    Q_OBJECT

private:
    /**
     * @brief Init header fields and default column widths
     */
    void initHeaderData();

    /**
     * @brief Init table data using the printer list file
     * @param path Path to local printer list
     */
    void initTableData(std::string path = "");

    /**
     * @brief Vector storing column widths
     */
    std::vector<int> *columnWidths;

    /**
     * @brief Vector holding header fields
     */
    std::vector<std::string> *headerFields;

    /**
     * @brief Vector holding the complete table data
     */
    std::vector<PrinterModel> *tableData;

    /**
     * @brief Vector holding only visible table rows
     */
    std::vector<PrinterModel> *visibleData;

    ConfigUtility *cfg;
    CupsUtility *cupsUtility;
    PrinterParser *parser;
    SMBUtility *smbUtility;

    int selection = -1;

    /**
     * @brief Get the currently selected printers
     * @return Selected printer model instance
     */
    PrinterModel getSelectedPrinter();

    /**
     * @brief Load column widths from config
     */
    void loadColumnWidths();

    /**
     * @brief Store column widths to config
     */
    void storeColumnWidths();

public:

    /**
     * @brief Enum storing sort modes
     */
    enum Mode {NONE, ASC, DSC};

    /**
     * @brief Add the currently selected printer using the cups util
     * @param driver Optional driver
     * @return Result string
     */
    Q_INVOKABLE bool addSelectedPrinter(QString driver);

    /**
     * @brief Clear the table filter
     */
    Q_INVOKABLE void clearFilter();

    /**
     * @brief Apply a non case sensitive filter to the table data using the given string
     * @param filterString Filter string
     */
    Q_INVOKABLE void filter(QString filterString);

    /**
     * @brief Get the background color of a table cell by row and column index
     * @param row Cell row index
     * @param column Cell column index
     * @return Color as QVariant
     */
    Q_INVOKABLE bool isCellSelected(int row, int column);

    /**
     * @brief Get the role of the given column
     * @param column Column index
     * @return Role as QString
     */
    Q_INVOKABLE QString getColumnRole(int column);

    /**
     * @brief Get the width of the given column
     * @param column Column to get width of
     * @return Width as int
     */
    Q_INVOKABLE int getColumnWidth(int column);

    /**
     * @brief Handle clicks on a table cell
     * @param row Clicked row
     * @param column Clicked column
     */
    Q_INVOKABLE void handleItemClicked(int row, int column);

    /**
     * @brief Check if the smb host requires authentication
     * @return True if authentication is required, else false
     */
    Q_INVOKABLE bool needCredentialsForPrinterList();

    /**
     * @brief Read the printer definition list without authentication
     * @return True if successfully read, else false
     */
    Q_INVOKABLE bool readRemotePrinterListAnonymous();

    /**
     * @brief Read the printer definition with given credentials
     *
     * In case of failure, the functions throws an exception using the qt js engine.
     * @param user Username
     * @param password password
     * @return True if successfully read, else false
     */
    Q_INVOKABLE bool readRemotePrinterListAuthenticated(QString user, QString password, QString domain);

    /**
     * @brief Read the remote driver file using the given credentials
     * @param user Username
     * @param password Password
     * @param domain SMB Domain
     * @return True if successfully read, else false
     */
    Q_INVOKABLE bool readRemoteDriverFileAuthenticated(QString user, QString password, QString domain);

    /**
     * @brief Load local printer definitions
     * @return True if loaded successfully, else false
     */
    Q_INVOKABLE bool loadLocalPrinterDefinitions();

    /**
     * @brief Sort the printer table
     * @param columnIndex Column used for sorting
     * @param mode Sort mode
     */
    Q_INVOKABLE void sortTable(int columnIndex, int mode);

    /**
     * @brief Store the width of a column
     * @param column Column index
     * @param width Width
     */
    Q_INVOKABLE void storeColumnWidth(int column, int width);

    /**
     * @brief Constructor
     * @param parent Parent object
     */
    explicit TableModel(QObject *parent = nullptr);

    /**
     * @brief Get table column widths as vector
     * @return  Columns widths as vector
     */
    Q_INVOKABLE std::vector<int> getColumnWidths();

    Q_INVOKABLE QVariant getSelectedQueueName();
    Q_INVOKABLE QVariant getSelectedServer();
    Q_INVOKABLE QVariant getSelectedLocation();
    Q_INVOKABLE QVariant getSelectedDriver();
    Q_INVOKABLE QVariant getSelectedFormat();
    Q_INVOKABLE QVariant getSelectedColor();
    Q_INVOKABLE QVariant getSelectedDuplex();
    Q_INVOKABLE QVariant getSelectedStaple();
    Q_INVOKABLE QVariant getSelectedPunch();
    Q_INVOKABLE QVariant getSelectedBooklet();
    Q_INVOKABLE QVariant getSelectedScanner();
    Q_INVOKABLE QVariant getSelectedOpts();
    Q_INVOKABLE QVariant getSelectedComment();
    Q_INVOKABLE QVariant getSelectedInstallCmd();
    Q_INVOKABLE QVariant getSelectedInstallCmd(QString printer);

    int getSortNone();
    int getSortAsc();
    int getSortDsc();

    Q_INVOKABLE QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;

    int rowCount(const QModelIndex & = QModelIndex()) const override;

    int columnCount(const QModelIndex & = QModelIndex()) const override;

    QVariant data(const QModelIndex &index, int role) const override;

    QHash<int, QByteArray> roleNames() const override;

    //Sort constants
    Q_PROPERTY(int SORT_NONE READ getSortNone CONSTANT)
    Q_PROPERTY(int SORT_ASC READ getSortAsc CONSTANT)
    Q_PROPERTY(int SORT_DSC READ getSortDsc CONSTANT)
};
#endif // TABLEMODEL_H
